//
//  Copyright (c) 2005-2007
//  James N Knight
//
//  Permission to use, copy, modify, distribute and sell this software
//  and its documentation for any purpose is hereby granted without fee,
//  provided that the above copyright notice appear in all copies and
//  that both that copyright notice and this permission notice appear
//  in supporting documentation.  The authors make no representations
//  about the suitability of this software for any purpose.
//  It is provided "as is" without express or implied warranty.
//
//  
// See http://homepages.inf.ed.ac.uk/svijayak/publications/vijayakumar-NeuCom2005.pdf
// for the original publication of this algorithm.



#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/matrix_expression.hpp>
#include <boost/numeric/ublas/triangular.hpp>
#include <boost/numeric/ublas/io.hpp>

#include <algorithm>

#include <math.h>

#include "rf.hpp"

RF::RF()
{

}

RF::RF(Types::Input c) : 
    center(c), 
    z(c.size()),
    temp2(c.size()),
    localModel(c.size()),
    a_H(2),
    a_G(2),
    a_E(0), 
    H_temp(2)
{
    d_def = 25;
    lambda = .999;
    gamma = 1e-6;
    alpha_init = 250;
    meta = false;

    W = 1e-10;
    e_2 = 0;

    a_H.clear();
    a_G.clear();

}

RF::RF(Types::Input c, const boost::program_options::variables_map& vm) : 
    center(c), 
    z(c.size()),
    temp2(c.size()),
    localModel(c.size(),vm),
    a_H(2),
    a_G(2),
    a_E(0), 
    H_temp(2)
{

    d_def = vm["d_def"].as<Types::RParam>();
    lambda = vm["lambda"].as<Types::RParam>();
    gamma = vm["gamma"].as<Types::RParam>();
    alpha_init = vm["alpha"].as<Types::RParam>();
    meta = vm["meta"].as<bool>();

    if(meta){
        meta_alpha = vm["theta"].as<Types::RParam>();
    }
        
    W = 1e-10;
    e_2 = 0;

    a_H.clear();
    a_G.clear();
}

Types::OutputT RF::confidence(Types::Input x) const {
    
    Types::vector z;
    localModel.project(x,z);

    Types::Input derivative = localModel.check_derivatives();
    q = element_div(element_prod(z,derivative),localModel.get_a_zz());

    return a_pk*(1 + getActivation(x) * ublas::prec_inner_prod(z,q));


}
