//
//  Copyright (c) 2005-2007
//  James N Knight
//
//  Permission to use, copy, modify, distribute and sell this software
//  and its documentation for any purpose is hereby granted without fee,
//  provided that the above copyright notice appear in all copies and
//  that both that copyright notice and this permission notice appear
//  in supporting documentation.  The authors make no representations
//  about the suitability of this software for any purpose.
//  It is provided "as is" without express or implied warranty.
//
//  
// See http://homepages.inf.ed.ac.uk/svijayak/publications/vijayakumar-NeuCom2005.pdf
// for the original publication of this algorithm.



#ifndef TYPES_HPP
#define TYPES_HPP

#include <boost/numeric/ublas/vector.hpp>
#include <boost/numeric/ublas/matrix.hpp>
#include <boost/numeric/ublas/triangular.hpp>
#include <boost/numeric/ublas/symmetric.hpp>
#include <boost/call_traits.hpp>


namespace ublas = boost::numeric::ublas;

/**
 * Define some default types for use in the LWPR program.  This will make it easier to 
 * make this code generic later.
 */

namespace Types {

    /** default type for real values */
    typedef double RValue;

    /** type for real values into functions */
    typedef boost::call_traits<RValue>::param_type RValueP;

    /** default type for real value parameters */
    typedef double RParam;

    /**
     * a ublas::vector is used to pass input vectors around.
     */
    typedef ublas::vector<RValue> vector; 
    
    /**
     * Use boost::call_traits to define the correct type for passing InputT into a function
     */
    typedef boost::call_traits<vector>::param_type Input;

    /**
     * a double is used for output of the predictor
     */
    typedef RValue OutputT;
    
    /**
     * Use boost::call_traits to define the correct type for passing OutputT into a function
     */
    typedef boost::call_traits<OutputT>::param_type Output;


    /** The type for activations */
    typedef RValue Activation;

    /** The type for matrices */
    typedef ublas::matrix<RValue,ublas::column_major> Matrix;

    /** The type for distance matrices */
    typedef ublas::symmetric_matrix<RValue,ublas::upper> SymMatrix;

    /** The type for Cholesky Decomposition of distance Matrix **/
    typedef ublas::triangular_matrix<RValue,ublas::upper> UTMatrix;


};

#endif
